# Guide for Flask AIWAF settings
# Add these to your Flask app config
# app.config['AIWAF_RATE_WINDOW'] = 10
# app.config['AIWAF_RATE_MAX'] = 20
# app.config['AIWAF_RATE_FLOOD'] = 40
# app.config['AIWAF_MIN_FORM_TIME'] = 1.0
