default_app_config = "aiwaf.django.apps.AiwafConfig"

__version__ = "1.0.6"

# Note: Middleware classes are available from aiwaf.django.middleware
# Import them only when needed to avoid circular imports during Django app loading
